#ifndef __TCPSERVER_H__
#define __TCPSERVER_H__

#include "config.h"

#include <stdio.h>
#include <string>
#include <time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <netdb.h>
#include <fcntl.h>
#include <errno.h>

#define TCP_SELECT_TIMEOUT 100

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#endif

// TCP/IP server to push data to the frontend.
// Mostly stolen from my Dominia code... I knew that would come in
// useful someday in a real context.

class TcpServer {
public:
    TcpServer();
    ~TcpServer();

    int Valid() { return sv_valid; };

    int Setup(unsigned int in_max_clients, short int in_port, const char *in_allowed);

    void Kill(int in_fd);

    int Poll();

    void Send(int in_fd, const char *in_data);

    void SendToAll(const char *in_data);

    void Shutdown();

    char *FetchError() { return errstr; };

protected:
    int Accept();

    char errstr[1024];

    // Active server
    int sv_valid;

    unsigned int max_clients;

    // Server info
    short int port;
    char hostname[MAXHOSTNAMELEN];

    const char *allowed;

    // Socket items
    unsigned int serv_fd;
    struct sockaddr_in serv_sock;
    struct hostent *serv_host;

    // Master list of Fd's
    fd_set server_fds;
    // Read/except fd's
    fd_set read_fds;
    fd_set except_fds;

    fd_set client_fds;

    unsigned int max_fd;

};

#endif
